@@ -29,15 +29,6 @@ from numpy.linalg import inv, LinAlgError
 def wmw_auc(y, r):
     """
     Compute the AUC by using the Wilcoxon-Mann-Whitney formula.
-    
-    Input
-    
-    * *y* - classes (two classes +1 and -1) [1D numpy array integer]
-    * *r* - real-valued prediction          [1D numpy array float]
-    
-    Output
-    
-    * wmwAUC
     """
     
     if y.shape[0] != r.shape[0]:
@@ -61,15 +52,6 @@ def wmw_auc(y, r):
 def mcc(y, p):
     """
     Compute the Matthews Correlation Coefficient (MCC).
-
-    MCC = ((tp*tn)-(fp*fn)) / sqrt((tp+fn)*(tp+fp)*(tn+fn)*(tn+fp))
-
-    Input
-      * *y* - classes    (two classes +1 and -1) [1D numpy array integer]
-      * *p* - prediction (two classes +1 and -1) [1D numpy array integer]
-
-    Output
-      * MCC
     """
 
     if y.shape[0] != p.shape[0]:
@@ -111,7 +93,8 @@ def dot3(a1, M, a2):
     
 
 class Dlda:
-    """Diagonal Linear Discriminant Analysis.
+    """
+    Diagonal Linear Discriminant Analysis.
 
     Example:
     
@@ -122,41 +105,41 @@ class Dlda:
     ...              [1.3, 2.2, 3.5, 1.0],  # third sample
     ...              [1.4, 2.1, 3.2, 2.0]]) # fourth sample
     >>> ytr = array([1, -1, 1, -1])         # classes
-    >>> mydlda = Dlda()                     # initialize dlda class
-    >>> xtr_std = data_standardize(xtr)     # standardize the training dataset
-    >>> mydlda.compute(xtr_std, ytr)        # compute dlda
+    >>> mydlda = Dlda(nf = 2)                     # initialize dlda class
+    >>> mydlda.compute(xtr, ytr)        # compute dlda
     1
-    >>> mydlda.predict(xtr_std)             # predict dlda model on training data
+    >>> mydlda.predict(xtr)             # predict dlda model on training data
     array([ 1, -1,  1, -1])
     >>> xts = array([4.0, 5.0, 6.0, 7.0])   # test point
     >>> mydlda.predict(xts)                 # predict dlda model on test point
     -1
     >>> mydlda.realpred                     # real-valued prediction
-    -45.32292203746583
-    >>> mydlda.weights(xtr_std, ytr)        # compute weights on training data
-    array([  3.00000000e+00,   3.00000000e+00,   9.32587341e-15,
-             2.61756029e+00])
+    -21.999999999999954
+    >>> mydlda.weights(xtr, ytr)        # compute weights on training data
+    array([  2.13162821e-14,   0.00000000e+00,   0.00000000e+00,   4.00000000e+00])
     """
-
    
     def __init__(self, nf = 0, tol = 10, overview = False, bal = False):
-        """Initialize Dlda class.
+        """
+        Initialize Dlda class.
         
-        Input:
-        
-          * *nf* - [integer] the number of the best features that we want
-            to use in the model (1 <= nf >= #features). If nf = 0 the
-            system stops at a number of features corresponding to a peak
-            of accuracy
+        :Parameters:
+          nf : int (1 <= nf >= #features)
+            the number of the best features that you want to use in
+            the model. If nf = 0 the system stops at a number of features
+            corresponding to a peak of accuracy
 
-          * *tol* - [integer] in case of nf = 0 it's the number of steps
+          tol : int
+            in case of nf = 0 it's the number of steps
             of classification to be calculated after the peak to avoid a
             local maximum
 
-          * *overview* - [bool] set True to print informations about the
+          overview : bool
+            set True to print informations about the
             accuracy of the classifier at every step of the compute
 
-          * *bal* - [bool] set True if it's reasonable to consider the
+          bal : bool
+            set True if it's reasonable to consider the
             unbalancement of the test set similar to the one of the
             training set
         """
@@ -222,36 +205,23 @@ class Dlda:
                 
         self.__b = BestB
                   
-#####     Here you find an implementadion of Eq.3 Pg.2
-#####     for offset (b) computation.
-        
-#         xp = self.__x[where (self.__y == 1)]
-#         xm = self.__x[where (self.__y == -1)]
-#         medpos = mean(xp[:, self.__A], axis = 0)
-#         medneg = mean(xm[:, self.__A], axis = 0)
-        
-#         if self.__bal == False:
-#             self.__b =  - dot(self.__WA.transpose() , 0.5*(medpos + medneg))
-#         else: 
-#             self.__b = log(self.__piP * 1.0 / self.__piN * 1.0) \
-#                        - dot(self.__WA.transpose(), 0.5 * (medpos + medneg))
-
-          
-
     def __choose_model(self):
         """With a l.o.o. classification verify which model gives the
         best accuracy.
         """
             
         tmp = ones((self.__K.shape[0], self.__K.shape[1]), dtype = int8)
-        tmp[:-1, :-1] = self.__Kmask   
+        tmp[:-1, :-1] = self.__Kmask
         tmp[-1, : - (len(self.__A) - self.__m_code)] = tmp[:-(len(self.__A) - self.__m_code), -1] = 0
         mask_sameblock = tmp.copy()
         tmp[-1, :-1] = tmp[:-1, -1] = 0
         mask_otherblock = tmp.copy()
-        
-        acc_ob, mcc_ob, auc_ob = self.__check_model(mask_otherblock)
-        acc_sb, mcc_sb, auc_sb = self.__check_model(mask_sameblock)
+
+        try:
+            acc_ob, mcc_ob, auc_ob = self.__check_model(mask_otherblock)
+            acc_sb, mcc_sb, auc_sb = self.__check_model(mask_sameblock)
+        except:
+            return 0
         
         if mcc_ob > mcc_sb:
             self.__Kmask = mask_otherblock
@@ -259,14 +229,14 @@ class Dlda:
             self.__m_code = len(self.__A) - 1
             if self.__overview == True:        
                 print 'With', len(self.__A), 'features the accuracy on training data is:', \
-                      acc_ob * 100, '% and the MCC value is', mcc_ob, "e auc =",auc_ob ,"OB", self.__b
+                      acc_ob * 100, '%, the MCC value is', mcc_ob, "and auc =",auc_ob
                 
         else:
             self.__Kmask = mask_sameblock
             self.__checkstop(mcc_sb)
             if self.__overview == True:        
                 print 'With', len(self.__A), 'features the accuracy on training data is:', \
-                      acc_sb * 100, '% and the MCC value is', mcc_sb, "e auc =",auc_sb ,"SB", self.__b
+                      acc_sb * 100, '%, the MCC value is', mcc_sb, "and auc =",auc_sb
 
     
     def __check_model(self, mask):
@@ -275,7 +245,6 @@ class Dlda:
         
         See Table1 Pg.2
         """
-
         p_mcc = zeros(self.__x.shape[0])
         rp_auc = zeros(self.__x.shape[0])
         n_right = 0
@@ -287,14 +256,20 @@ class Dlda:
             xsf = xf[s,:]
             ys = self.__y[s]
             ytest = self.__y[i]
-            K = cov(xsf.transpose(), bias = 1) * mask
-
+            try:
+                K = cov(xsf.transpose(), bias = 1) * mask
+            except:
+                return 0
+            
             medpos = mean(xsf[where(ys == 1), :][0], axis = 0)
             medneg = mean(xsf[where(ys == -1), :][0], axis = 0)
-            d = medpos - medneg 
-            w = dot(inv(K), d)
-            pred = dot(self.__x[i,self.__A], w) - self.__b
-
+            d = medpos - medneg
+            try: 
+                w = dot(inv(K), d)
+            except LinAlgError:
+                w = dot(pinv(K), d)
+            
+            pred = dot(self.__x[i,self.__A], w) - self.__b    
             rp_auc[i] = pred
             
             if pred >= 0.0:
@@ -320,7 +295,10 @@ class Dlda:
         if self.__K == None:
             self.__K = array([[cov(self.__x[:,BF], bias = 1)]])
             self.__d = self.__compute_d(BF)
-            self.__WA = dot(inv(self.__K), self.__d)
+            try:
+                self.__WA = dot(inv(self.__K), self.__d)
+            except:
+                self.__WA = dot(pinv(self.__K), self.__d)
         else:
             res = self.__compute_WA(BF)
             self.__WA = res[0]
@@ -353,8 +331,11 @@ class Dlda:
         
         d = self.__compute_d(j)
         K = self.__update_K(j)
-        ### Attention: adding a new feature we don't have info about the mask so we use the whole cov matrik K
-        WA = dot(inv(K),d)  
+        ### NB: adding a new feature we don't have info about the mask so we use the whole cov matrik K
+        try:
+            WA = dot(inv(K),d)
+        except:
+            WA = dot(pinv(K),d)
         return [WA,d,K]
 
 
@@ -381,7 +362,11 @@ class Dlda:
         """
         
         if M > (self.__peak + 1e-3): # Don't update under 1e-3 over the peak
-            self.__WA_stored = dot(inv(self.__K*self.__Kmask),self.__d)
+            try:
+                self.__WA_stored = dot(inv(self.__K*self.__Kmask),self.__d)
+            except:
+                self.__SingularMatrix = True
+                return 0
             self.__b_stored = self.__b
             self.__A_stored = self.__A[:]
             self.__cont = 0
@@ -416,78 +401,74 @@ class Dlda:
                 bestval = None
                 bestfeat = None
                 for j in self.__AC:
-                    try:
-                        res_j = self.__compute_j(j)
-                    except LinAlgError:
-                        print 'Feature', j, 'generates a singular matrix after',\
-                              len(self.__A), 'features already chosen'
-                    else:
-                        val_j = res_j
-                        if val_j >= bestval:
-                            bestval = val_j
-                            bestfeat = j
+                    res_j = self.__compute_j(j)
+                    val_j = res_j
+                    if val_j >= bestval:
+                        bestval = val_j
+                        bestfeat = j
                             
                 if bestfeat == None:  # If all the features generate a singular matrix the compute returns 0
                     return 0
                 else:
                     self.__addfeat(bestfeat)
                     self.__choose_model()
+            try:
+                self.__WA = dot(inv(self.__K*self.__Kmask),self.__d)
+            except:
+                self.__WA = dot(pinv(self.__K*self.__Kmask),self.__d)
 
-            self.__WA = dot(inv(self.__K*self.__Kmask),self.__d)
             if self.__overview == True:
-                print "PESI nf =", self.__nf ,self.__WA
+                print "Weights for ", self.__nf, "features: " ,self.__WA
                 print 'This model is going to use', len(self.__A), 'features'
 
-            return 1
-
         # IF USE AUTOSTOP 
         if self.__nf == 0:
             while ((len(self.__AC) > 0) and (self.__cont < self.__tol)):
                 bestval = None
                 bestfeat = None
                 for j in self.__AC:
-                    try:
-                        res_j = self.__compute_j(j)
-                    except LinAlgError:
-                        print 'Feature', j, 'generates a singular matrix after', \
-                              len(self.__A), 'features already chosen'
-                    else:
-                        val_j = res_j
-                        if val_j >= bestval:
-                            bestval = val_j
-                            bestfeat = j
-                            
+
+                    res_j = self.__compute_j(j)
+                    val_j = res_j
+                    if val_j >= bestval:
+                        bestval = val_j
+                        bestfeat = j
+
+                print bestfeat
                 if bestfeat == None:  # If all the features generate a singular matrix the compute returns 0
                     return 0
+
                 else:
                     self.__addfeat(bestfeat)
                     self.__choose_model()
-        
+            
             self.__WA = self.__WA_stored
             self.__b = self.__b_stored
             self.__A = self.__A_stored
             
             if self.__overview == True:
-                print "PESI nf=0",self.__WA
+                print "Weights for ",len(self.__A),"features: ", self.__WA
                 print 'This model is going to use', len(self.__A), 'features'
 
-            return 1
-
-
     def compute (self, x, y, mf = 0):
-        """Compute Dlda model.
+        """
+        Compute Dlda model.
 
-        Initialize array of alphas a.
-       
-        Input
-        
-          * *x*  - [2D numpy array float] (sample x feature) training data
-          * *y*  - [1D numpy array integer] (two classes) classes
-          * *mf* - [integer] (More Features) number of classification steps to be calculated
+        :Parameters:  
+          x : 2d ndarray float (samples x feats)
+            training data
+          y : 1d ndarray integer (-1 or 1)
+            classes
+          mf : int
+            number of classification steps to be calculated
             more on a model already computed
 
-        Output
-          * 1
+        :Returns:
+          1
+
+        :Raises:
+          LinAlgError
+            if x is singular matrix
         """
         
         if (self.__nf == 0) or (self.__computed == False):
@@ -536,16 +517,21 @@ class Dlda:
         return 1
         
     def predict (self, p):
-        """Predict Dlda model on test point(s).
+        """
+        Predict Dlda model on test point(s).
 
-        Input
-        
-          * *p* - [1D or 2D numpy array float] test point(s)
+        :Parameters:
+          p : 1d or 2d ndarray float (sample(s) x feats)
+            test sample(s)
 
-        Output
+        :Returns:
+          cl : integer or 1d numpy array integer
+            class(es) predicted
+
+        :Attributes:
+          self.realpred : float or 1d numpy array float
+            real valued prediction
         
-          * *cl* - [integer or 1D numpy array integer] class(es) predicted
-          * *self.realpred* - [1D numpy array float] real valued prediction
         """
         if self.__computed == False:
             raise StandardError("Dlda model not computed yet")
@@ -570,18 +556,20 @@ class Dlda:
 
 
     def weights (self, x, y):
-        """Return feature weights.
-        
-        Input
-        
-          * *x* - [2D numpy array float] (sample x feature) training data
-          * *y* - [1D numpy array integer] (two classes, 1 or -1) classes
+        """
+        Return feature weights.
 
-        Output
+        :Parameters:
+          x : 2d ndarray float (samples x feats)
+            training data
+          y : 1d ndarray integer (-1 or 1)
+            classes
         
-          * *fw* - [1D numpy array float] feature weights (they are gonna be
+        :Returns:
+          fw :  1d ndarray float
+            feature weights, they are going to be
             > 0 for the features chosen for the classification and = 0 for
-            all the others)
+            all the others
         """
     
         self.compute(x, y, 0)
